@@ -33,22 +33,6 @@
 static stats_entry_t *stats = NULL;
 volatile int stats_shm_handle = NOT_INITIALIZED;
 
-
-static inline void atomic_add(uint64_t *x, uint64_t i)
-{
-    uint64_t old;
-    uint64_t new;
-    asm volatile ("   lg      %0,%2\n"
-                  "0: lgr     %1,%0\n"
-                  "   agr     %1,%3\n"
-                  "   csg     %0,%1,%2\n"
-                  "   jl      0b"
-                 :"=&d" (old), "=&d"(new), "=Q"(*x)
-                 :"d"(i), "Q"(*x)
-                 :"cc", "memory");
-}
-
-
 /* open shared memory segment
  * Arguments:
  * @user: if user is -1 stats_mmap will open the shared memory segent of the same
@@ -63,24 +47,36 @@ static inline void atomic_add(uint64_t *x, uint64_t i)
 int stats_mmap(int user)
 {
 	char shm_id[NAME_LENGHT];
+	struct stat stat_buf;
 
 	if (stats == NULL) {
 		sprintf(shm_id, "icastats_%d",
 			user == -1 ? geteuid() : (uid_t)user);
 
-		stats_shm_handle = shm_open(shm_id, O_CREAT | O_RDWR,
+		stats_shm_handle = shm_open(shm_id, O_RDWR,  S_IRUSR | S_IWUSR);
+		if (stats_shm_handle == NOT_INITIALIZED)
+			stats_shm_handle = shm_open(shm_id, O_CREAT | O_RDWR,
 						    S_IRUSR | S_IWUSR);
 
 		if (stats_shm_handle == NOT_INITIALIZED)
 			return -1;
 
 		if (user > 0 && geteuid() == 0) {
-			if (fchown(stats_shm_handle, user, user) == -1)
+			if (fchown(stats_shm_handle, user, user) == -1) {
+				close(stats_shm_handle);
 				return -1;
+			}
 		}
 
-		if (ftruncate(stats_shm_handle, STATS_SHM_SIZE) == -1)
+		if (fstat(stats_shm_handle, &stat_buf)) {
+			close(stats_shm_handle);
 			return -1;
+		}
+
+		if (ftruncate(stats_shm_handle, STATS_SHM_SIZE) == -1) {
+			close(stats_shm_handle);
+			return -1;
+		}
 
 		stats = (stats_entry_t *) mmap(NULL, STATS_SHM_SIZE, PROT_READ |
 						 PROT_WRITE, MAP_SHARED,
@@ -90,6 +86,9 @@ int stats_mmap(int user)
 			stats = NULL;
 			return -1;
 		}
+
+		if (stat_buf.st_size != STATS_SHM_SIZE)
+			memset(stats, 0, STATS_SHM_SIZE);
 	}
 	return 0;
 }
@@ -142,18 +141,87 @@ uint64_t stats_query(stats_fields_t field, int hardware, int direction)
 			return stats[field].dec.sw;
 }
 
+static uint64_t calc_summary(stats_fields_t start, unsigned int num,
+		             int hardware, int direction)
+{
+	unsigned int i;
+	uint64_t sum = 0;
+
+	for (i = 0; i < num; i++)
+		sum += stats_query(start + i, hardware, direction);
+
+	return sum;
+}
+
 /* Returns the statistic data in a stats_entry_t array
  * @entries - Needs to be a array of size ICA_NUM_STATS.
  */
-
 void get_stats_data(stats_entry_t *entries)
 {
 	unsigned int i;
-	for(i = 0;i<ICA_NUM_STATS; i++){
-		entries[i].enc.hw = stats_query(i, ALGO_HW, ENCRYPT);
-		entries[i].enc.sw = stats_query(i, ALGO_SW, ENCRYPT);
-		entries[i].dec.hw = stats_query(i, ALGO_HW, DECRYPT);
-		entries[i].dec.sw = stats_query(i, ALGO_SW, DECRYPT);
+	for(i = 0; i < ICA_NUM_STATS; i++) {
+		switch (i) {
+		case ICA_STATS_AES_ECB:
+		case ICA_STATS_AES_CBC:
+		case ICA_STATS_AES_OFB:
+		case ICA_STATS_AES_CFB:
+		case ICA_STATS_AES_CTR:
+		case ICA_STATS_AES_CMAC:
+		case ICA_STATS_AES_GCM:
+			entries[i].enc.hw = calc_summary(i + 1, 3,
+					                 ALGO_HW, ENCRYPT);
+			entries[i].enc.sw = calc_summary(i + 1, 3,
+					                 ALGO_SW, ENCRYPT);
+			entries[i].dec.hw = calc_summary(i + 1, 3,
+					                 ALGO_HW, DECRYPT);
+			entries[i].dec.sw = calc_summary(i + 1, 3,
+					                 ALGO_SW, DECRYPT);
+			break;
+
+		case ICA_STATS_AES_XTS:
+			entries[i].enc.hw = calc_summary(i + 1, 2,
+					                 ALGO_HW, ENCRYPT);
+			entries[i].enc.sw = calc_summary(i + 1, 2,
+					                 ALGO_SW, ENCRYPT);
+			entries[i].dec.hw = calc_summary(i + 1, 2,
+					                 ALGO_HW, DECRYPT);
+			entries[i].dec.sw = calc_summary(i + 1, 2,
+					                 ALGO_SW, DECRYPT);
+			break;
+
+		case ICA_STATS_RSA_ME:
+		case ICA_STATS_RSA_CRT:
+			entries[i].enc.hw = calc_summary(i + 1, 4,
+					                 ALGO_HW, ENCRYPT);
+			entries[i].enc.sw = calc_summary(i + 1, 4,
+					                 ALGO_SW, ENCRYPT);
+			entries[i].dec.hw = calc_summary(i + 1, 4,
+					                 ALGO_HW, DECRYPT);
+			entries[i].dec.sw = calc_summary(i + 1, 4,
+					                 ALGO_SW, DECRYPT);
+			break;
+
+		case ICA_STATS_ECDH:
+		case ICA_STATS_ECDSA_SIGN:
+		case ICA_STATS_ECDSA_VERIFY:
+		case ICA_STATS_ECKGEN:
+			entries[i].enc.hw = calc_summary(i + 1, 8,
+					                 ALGO_HW, ENCRYPT);
+			entries[i].enc.sw = calc_summary(i + 1, 8,
+					                 ALGO_SW, ENCRYPT);
+			entries[i].dec.hw = calc_summary(i + 1, 8,
+					                 ALGO_HW, DECRYPT);
+			entries[i].dec.sw = calc_summary(i + 1, 8,
+					                 ALGO_SW, DECRYPT);
+			break;
+
+		default:
+			entries[i].enc.hw = stats_query(i, ALGO_HW, ENCRYPT);
+			entries[i].enc.sw = stats_query(i, ALGO_SW, ENCRYPT);
+			entries[i].dec.hw = stats_query(i, ALGO_HW, DECRYPT);
+			entries[i].dec.sw = stats_query(i, ALGO_SW, DECRYPT);
+			break;
+		}
 	}
 }
 
@@ -277,14 +345,14 @@ void stats_increment(stats_fields_t field, int hardware, int direction)
 
 	if(direction == ENCRYPT)
 		if (hardware == ALGO_HW)
-			atomic_add(&stats[field].enc.hw, 1);
+			__sync_add_and_fetch(&stats[field].enc.hw, 1);
 		else
-			atomic_add(&stats[field].enc.sw, 1);
+			__sync_add_and_fetch(&stats[field].enc.sw, 1);
 	else
 		if (hardware == ALGO_HW)
-			atomic_add(&stats[field].dec.hw, 1);
+			__sync_add_and_fetch(&stats[field].dec.hw, 1);
 		else
-			atomic_add(&stats[field].dec.sw, 1);
+			__sync_add_and_fetch(&stats[field].dec.sw, 1);
 }
 #endif
 
