@@ -30,6 +30,10 @@
 #include "s390_crypto.h"
 #include "s390_ctr.h"
 
+#if OPENSSL_VERSION_PREREQ(3, 0)
+extern OSSL_LIB_CTX *openssl_libctx;
+#endif
+
 #define AES_BLOCK_SIZE 16
 #define GCM_RECOMMENDED_IV_LENGTH 12
 
@@ -136,7 +140,7 @@ static inline int s390_aes_gcm(unsigned int fc, const unsigned char *in_data,
 	if (rc)
 		return rc;
 
-	stats_increment(ICA_STATS_AES_GCM,
+	stats_increment(ICA_STATS_AES_GCM_128 + aes_directed_fc_stats_ofs(fc),
 			ALGO_HW,
 			(s390_kma_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
@@ -160,7 +164,8 @@ static inline int __s390_aes_ctrlist(unsigned int fc, unsigned long data_length,
 	if (rc)
 		return rc;
 
-	stats_increment(ICA_STATS_AES_CTR, ALGO_HW,
+	stats_increment(ICA_STATS_AES_CTR_128 + aes_directed_fc_stats_ofs(fc),
+			ALGO_HW,
 			 (s390_msa4_functions[fc].hw_fc &
 			 S390_CRYPTO_DIRECTION_MASK) ==
 			 0 ?ENCRYPT:DECRYPT);
@@ -308,12 +313,15 @@ static inline int s390_aes_ecb_sw(unsigned int function_code,
 	unsigned int direction;
 	unsigned int key_size = (function_code & 0x0f) *
 				sizeof(ica_aes_key_single_t);
+	int rc = 0;
 
 #ifdef ICA_FIPS
-	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+	if ((fips & ICA_FIPS_MODE) && (!openssl_in_fips_mode()))
 		return EACCES;
 #endif /* ICA_FIPS */
 
+	BEGIN_OPENSSL_LIBCTX(openssl_libctx, rc);
+
 	if (function_code & S390_CRYPTO_DIRECTION_MASK) {
 		AES_set_decrypt_key(keys, key_size * 8, &aes_key);
 		direction = AES_DECRYPT;
@@ -327,7 +335,10 @@ static inline int s390_aes_ecb_sw(unsigned int function_code,
 				&aes_key, direction);
 	}
 
-	return 0;
+	OPENSSL_cleanse(&aes_key, sizeof(aes_key));
+
+	END_OPENSSL_LIBCTX(rc);
+	return rc;
 }
 
 static inline int s390_aes_cbc_hw(unsigned int function_code,
@@ -372,12 +383,15 @@ static inline int s390_aes_cbc_sw(unsigned int function_code,
 	unsigned int direction;
 	unsigned int key_size = (function_code & 0x0f) *
 				sizeof(ica_aes_key_single_t);
+	int rc = 0;
 
 #ifdef ICA_FIPS
-	if ((fips & ICA_FIPS_MODE) && (!FIPS_mode()))
+	if ((fips & ICA_FIPS_MODE) && (!openssl_in_fips_mode()))
 		return EACCES;
 #endif /* ICA_FIPS */
 
+	BEGIN_OPENSSL_LIBCTX(openssl_libctx, rc);
+
 	if (function_code & S390_CRYPTO_DIRECTION_MASK) {
 		AES_set_decrypt_key(keys, key_size * 8, &aes_key);
 		direction = AES_DECRYPT;
@@ -388,7 +402,10 @@ static inline int s390_aes_cbc_sw(unsigned int function_code,
 	AES_cbc_encrypt(input_data, output_data, input_length,
 			&aes_key, (unsigned char *) iv, direction);
 
-	return 0;
+	OPENSSL_cleanse(&aes_key, sizeof(aes_key));
+
+	END_OPENSSL_LIBCTX(rc);
+	return rc;
 }
 
 static inline int s390_aes_ecb(unsigned int fc, unsigned long data_length,
@@ -410,7 +427,8 @@ static inline int s390_aes_ecb(unsigned int fc, unsigned long data_length,
 				     out_data);
 		hardware = ALGO_SW;
 	}
-	stats_increment(ICA_STATS_AES_ECB,
+
+	stats_increment(ICA_STATS_AES_ECB_128 + aes_directed_fc_stats_ofs(fc),
 			hardware,
 			(s390_kmc_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
@@ -437,7 +455,8 @@ static inline int s390_aes_cbc(unsigned int fc, unsigned long data_length,
 				     out_data);
 		hardware = ALGO_SW;
 	}
-	stats_increment(ICA_STATS_AES_CBC,
+
+	stats_increment(ICA_STATS_AES_CBC_128 + aes_directed_fc_stats_ofs(fc),
 			hardware, (s390_kmc_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
 			ENCRYPT:DECRYPT);
@@ -490,7 +509,8 @@ static inline int __s390_aes_cfb(unsigned int fc, unsigned long data_length,
 	if (rc)
 		return rc;
 
-	stats_increment(ICA_STATS_AES_CFB, ALGO_HW,
+	stats_increment(ICA_STATS_AES_CFB_128 + aes_directed_fc_stats_ofs(fc),
+			ALGO_HW,
 			(s390_kmc_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
 			ENCRYPT:DECRYPT);
@@ -583,7 +603,8 @@ static inline int __s390_aes_ofb(unsigned int fc, unsigned long input_length,
 	if (rc)
 		return rc;
 
-	stats_increment(ICA_STATS_AES_OFB, ALGO_HW,
+	stats_increment(ICA_STATS_AES_OFB_128 + aes_directed_fc_stats_ofs(fc),
+			ALGO_HW,
 			(s390_kmc_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
 			ENCRYPT:DECRYPT);
@@ -828,7 +849,8 @@ static inline int s390_aes_xts(unsigned int fc, unsigned long data_length,
 	if (rc)
 		return rc;
 
-	stats_increment(ICA_STATS_AES_XTS, ALGO_HW,
+	stats_increment(ICA_STATS_AES_XTS_128 + aes_directed_fc_stats_ofs(fc),
+			ALGO_HW,
 			(s390_kmc_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
 			ENCRYPT:DECRYPT);
