@@ -28,19 +28,12 @@
 #include "icastats.h"
 #include "s390_drbg.h"
 
-/*
- * On 31 bit systems we have to use the instruction STCKE while on 64 bit
- * systems we can use STCKF. STCKE uses a 16 byte buffer while STCKF uses
- * an 8 byte buffer.
- */
-#ifdef _LINUX_S390X_
 #define STCK_BUFFER  8
-#else
-#define STCK_BUFFER 16
-#endif
 
 /*
- * State handle for the global ica_drbg instantiation.
+ * State handle for the global ica_drbg instantiation that replaces
+ * the old prng implementation (if available) which feeds
+ * the ica_random_number_generate api,
  */
 ica_drbg_t *ica_drbg_global = ICA_DRBG_NEW_STATE_HANDLE;
 
@@ -77,6 +70,35 @@ static int s390_prng_seed(void *srv, unsigned int count);
 /* Constant */
 #define PRNG_BLK_SZ	8
 
+#if defined(NO_CPACF) && defined(ICA_FIPS)
+/**
+ * Generate random numbers using fips-approved random sources.
+ * /dev/random can be used in fips mode, but not /dev/urandom.
+ */
+static int s390_prng_fips(unsigned char *output_data, unsigned int output_length)
+{
+	const char *const rng_list[] = {
+			"/dev/prandom",
+			"/dev/hwrng",
+			"/dev/random" };
+	const size_t rng_list_len = sizeof(rng_list) / sizeof(rng_list[0]);
+	size_t priority, status;
+	FILE *fd;
+
+	for (priority = 0; priority < rng_list_len; priority++) {
+		fd = fopen(rng_list[priority], "r");
+		if (fd) {
+			status = fread(output_data, output_length, 1, fd);
+			fclose(fd);
+			if (status == 1)
+				return 0;
+		}
+	}
+
+	return EIO;
+}
+#endif
+
 int s390_prng_init(void)
 {
 	int rc = -1;
@@ -144,10 +166,10 @@ static int s390_add_entropy(void)
 		return ENOTSUP;
 
 	for (K = 0; K < 16; K++) {
-		s390_stck(entropy + 0 * STCK_BUFFER);
-		s390_stck(entropy + 1 * STCK_BUFFER);
-		s390_stck(entropy + 2 * STCK_BUFFER);
-		s390_stck(entropy + 3 * STCK_BUFFER);
+		s390_stckf_hw(entropy + 0 * STCK_BUFFER);
+		s390_stckf_hw(entropy + 1 * STCK_BUFFER);
+		s390_stckf_hw(entropy + 2 * STCK_BUFFER);
+		s390_stckf_hw(entropy + 3 * STCK_BUFFER);
 		if(s390_kmc(0x43, zPRNG_PB.ch, entropy, entropy,
 			      sizeof(entropy)) < 0) {
 			return EIO;
@@ -223,6 +245,14 @@ int s390_prng(unsigned char *output_data, unsigned int output_length)
 			return 0;
 	}
 
+#if defined(NO_CPACF) && defined(ICA_FIPS)
+	rc = s390_prng_fips(output_data, output_length);
+	if (rc == 0)
+		stats_increment(ICA_STATS_PRNG, ALGO_SW, ENCRYPT);
+
+	return rc;
+#endif
+
 #ifndef ICA_FIPS	/* Old prng code disabled with FIPS built. */
 	if (prng_switch)
 		rc = s390_prng_hw(output_data, output_length);
@@ -275,7 +305,7 @@ static int s390_prng_hw(unsigned char *random_bytes, unsigned int num_bytes)
 		num_bytes -= remainder;
 
 		for (i = 0; i < (num_bytes / STCK_BUFFER); i++)
-			s390_stck(random_bytes + i * STCK_BUFFER);
+			s390_stckf_hw(random_bytes + i * STCK_BUFFER);
 
 		rc = s390_kmc(S390_CRYPTO_PRNG, zPRNG_PB.ch, random_bytes,
 			      random_bytes, num_bytes);
@@ -286,7 +316,7 @@ static int s390_prng_hw(unsigned char *random_bytes, unsigned int num_bytes)
 
 		// If there was a remainder, we'll use an internal buffer to handle it.
 		if (!rc && remainder) {
-			s390_stck(last_dw);
+			s390_stckf_hw(last_dw);
 			rc = s390_kmc(S390_CRYPTO_PRNG, zPRNG_PB.ch, last_dw,
 				      last_dw, STCK_BUFFER);
 			if (rc > 0) {
